function  [This,Ok] = mysstatenonlin(This,Opt)
% mysstatenonlin [Not a public function] Steady-state solver for non-linear models.
%
% Backend IRIS function.
% No help provided.

% -IRIS Toolbox.
% -Copyright (c) 2007-2015 IRIS Solutions Team.

fixL = Opt.fixL;
fixG = Opt.fixG;
nameBlkL = Opt.nameBlkL;
nameBlkG = Opt.nameBlkG;
eqtnBlk = Opt.eqtnBlk;
blkFunc = Opt.blkFunc;
endogLInx = Opt.endogLInx;
endogGInx = Opt.endogGInx;
zeroLInx = Opt.zeroLInx;
zeroGInx = Opt.zeroGInx;

%--------------------------------------------------------------------------

Shift = 10;
nAlt = size(This.Assign,3);
Ok = true(1,nAlt);

doRefresh();

% Set the level and growth of optimal policy multipliers to zero. We must
% do this before checking for NaNs in fixed variables.
if Opt.zeromultipliers
    This.Assign(1,This.multiplier,:) = 0;
end

% Check for levels and growth rate fixed to NaNs.
doChkForNans();

x0 = [];
for iAlt = 1 : nAlt
    
    iOk = true;
    
    % Initialise levels
    %-------------------
    x = real(This.Assign(1,:,iAlt));
    % Level variables that are set to zero (shocks).
    x(zeroLInx) = 0;
    % Assign NaN level initial conditions. First, assign values from the
    % previous iteration, if they exist and option 'reuse=' is `true`.
    inx = isnan(x) & endogLInx;
    if Opt.reuse && any(inx) && ~isempty(x0)
        x(inx) = x0(inx);
        inx = isnan(x) & endogLInx;
    end
    % Then, if there still some NaNs left, use the option `'nanInit='`
    % to assign them.
    x(inx) = real(Opt.naninit);
    
    % Initialise growth rates
    %-------------------------
    dx = imag(This.Assign(1,:,iAlt));
    % Growth variables that are set to zero (all variables if `'growth='
    % false`.
    dx(zeroGInx) = 0;
    if any(~zeroGInx)
        % Assign NaN growth initial conditions. First, assign values from
        % the previous iteration, if they exist and option 'reuse=' is
        % `true`.
        inx = isnan(dx) & endogGInx;
        if Opt.reuse && any(inx) && ~isempty(dx0)
            dx(inx) = dx0(inx);
            inx = isnan(dx) & endogLInx;
        end
        % Then, if there still some NaNs left, use the option `'NaN='` to assign
        % them.
        dx(inx) = imag(Opt.naninit);
    end
    % Re-assign zero growth for log variables to 1.
    dx(This.IxLog & dx == 0) = 1;
        
    % Cycle over individual blocks
    %------------------------------
    nBlk = length(nameBlkL);
    for iBlk = 1 : nBlk
        if isempty(nameBlkL{iBlk}) && isempty(nameBlkG{iBlk})
            continue
        end

        xi = nameBlkL{iBlk};
        dxi = nameBlkG{iBlk};
        ixLogPlus = Opt.IxLogPlus(xi);
        ixLogMinus = Opt.IxLogMinus(xi);
        % Log growth rates are always positive.
        ixDLog = This.IxLog(dxi);
        ixLogPlus = [ixLogPlus,ixDLog]; %#ok<AGROW>
        ixLogMinus = [ixLogMinus,false(size(dxi))]; %#ok<AGROW>
        z0 = [x(xi),dx(dxi)];
        z0(ixLogPlus | ixLogMinus) = log(abs(z0(ixLogPlus | ixLogMinus)));
        
        % Test all equations in this block for NaNs and INfs.
        if Opt.warning
            check = blkFunc{iBlk}(x,dx);
            inx = isnan(check) | isinf(check);
            if any(inx)
                utils.warning('model:mysstatenonlin', ...
                    'This equation evaluates to NaN or Inf: ''%s''.', ...
                    This.eqtn{eqtnBlk{iBlk}(inx)});
            end
        end
        
        % Number of levels; this variables is used also within
        % `doObjFunc()`.
        nxi = length(xi);
        
        % Function handles to equations in this block.
        f = blkFunc{iBlk};
        
        % Solve the block.
        if Opt.ixAssign(iBlk)
            % Plain assignment
            %------------------
            % The vectors `xi` and `dxi` are each either empty or
            % scalar.
            z = [];
            y0 = f(x,dx);
            if ~isempty(xi)
                z = [z,y0]; %#ok<AGROW>
            end
            if ~isempty(dxi)
                xk = x;
                xk(~This.IxLog) = x(~This.IxLog) + Shift*dx(~This.IxLog);
                xk(This.IxLog) = x(This.IxLog) .* dx(This.IxLog).^Shift;
                yk = f(xk,dx);
                % `ixDLog` is a scalar in a plain assignment block.
                if ixDLog
                    z = [z,(yk/y0)^(1/Shift)]; %#ok<AGROW>
                else
                    z = [z,(yk-y0)/Shift]; %#ok<AGROW>
                end
            end
            exitFlag = 1;

        else
            % System of equations
            %---------------------
            solverName = Opt.solver;
            if isfunc(solverName)
                solverName = func2str(solverName);
            end
            switch lower(solverName)
                case 'lsqnonlin'
                    [z,~,~,exitFlag] = ...
                        lsqnonlin(@doObjFunc,z0,[],[],Opt.optimset);
                    if exitFlag == -3
                        exitFlag = 1;
                    end
                case 'fsolve'
                    [z,~,exitFlag] = fsolve(@doObjFunc,z0,Opt.optimset);
                    if exitFlag == -3
                        exitFlag = 1;
                    end
            end
            z(abs(z) <= Opt.optimset.TolX) = 0; %#ok<AGROW>
            z(ixLogPlus) = exp(z(ixLogPlus)); %#ok<AGROW>
            z(ixLogMinus) = -exp(z(ixLogMinus)); %#ok<AGROW>
        end
        
        x(xi) = z(1:nxi);
        dx(dxi) = z(nxi+1:end);
        iOk = ~any(isnan(z)) && double(exitFlag) > 0;
    end

    This.Assign(1,:,iAlt) = x + 1i*dx;
    
    % Check for zero log variables.
    iOk = mychk(This,iAlt,'log') && iOk;
    
    % TODO: Report more details on which equations and which variables failed.
    if Opt.warning && ~iOk
        utils.warning('model:mysstatenonlin', ...
            'Steady state inaccurate or not returned for some variables.');
    end
    
    % Store the current values to initialise the next parameterisation.
    x0 = x;
    dx0 = dx;
    
    Ok(iAlt) = iOk;
end

doRefresh();


% Nested functions...


%**************************************************************************


    function doRefresh()
        if ~isempty(This.Refresh) && Opt.refresh
            This = refresh(This);
        end
    end % doRefresh()


%**************************************************************************


    function Y = doObjFunc(P)
        % doobjfunc  This is the objective function for the solver. Evaluate the
        % equations twice, at time t and t+Shift.

        % Delogarithmize log variables (variables in steady-state equations
        % are expected in original levels).
        P(ixLogPlus) = exp(P(ixLogPlus));
        P(ixLogMinus) = -exp(P(ixLogMinus));

        % Split the vector of unknows into levels and growth rates; `nxi` is the
        % number of levels.
        x(xi) = P(1:nxi);
        dx(dxi) = P(nxi+1:end);
        
        % Refresh all dynamic links.
        if ~isempty(This.Refresh)
            doRefresh();
        end
        
        Y = f(x,dx);
        if any(dxi)
            % Some growth rates need to be calculated. Evaluate the model equations at
            % time t and t+Shift if at least one growth rate is needed.
            xk = x;
            xk(~This.IxLog) = x(~This.IxLog) + Shift*dx(~This.IxLog);
            xk(This.IxLog) = x(This.IxLog) .* dx(This.IxLog).^Shift;
            Y = [Y;f(xk,dx)];
        end

        
        function doRefresh()
            % dorefresh  Refresh dynamic links in each iteration.
            This.Assign(1,:,iAlt) = x + 1i*dx;
            This = refresh(This,iAlt);
            x = real(This.Assign(1,:,iAlt));
            dx = imag(This.Assign(1,:,iAlt));
            dx(This.IxLog & dx == 0) = 1;
        end % doRefresh()
        
        
    end % doObjFunc()


%**************************************************************************


    function doChkForNans()
        % Check for levels fixed to NaN.
        fixLevelInx = false(1,length(This.name));
        fixLevelInx(fixL) = true;
        nanSstate = any(isnan(real(This.Assign)),3) & fixLevelInx;
        if any(nanSstate)
            utils.error('model:mysstatenonlin', ...
                ['Cannot fix steady-state level for this variable ', ...
                'because it is NaN: ''%s''.'], ...
                This.name{nanSstate});
        end
        % Check for growth rates fixed to NaN.
        fixGrowthInx = false(1,length(This.name));
        fixGrowthInx(fixG) = true;
        nanSstate = any(isnan(imag(This.Assign)),3) & fixGrowthInx;
        if any(nanSstate)
            utils.error('model:mysstatenonlin', ...
                ['Cannot fix steady-state growth for this variable ', ...
                'because it is NaN: ''%s''.'], ...
                This.name{nanSstate});
        end
    end % dochkfornans()


end
